import asyncio
import logging
from potnanny.models.device import Device
from potnanny.controllers.pipeline import Pipeline
from potnanny.utils import utcnow
from .ble import BLEInterface
from .gpio import GPIOInterface
from .parse import Parser


logger = logging.getLogger(__name__)


class Poller:

    def __init__(self, *args, **kwargs):
        self.convert_c = False
        self.leaf_offset = -2
        self.devices = []
        self.now = utcnow()
        self._kwargs = kwargs

        for k,v in kwargs.items():
            if hasattr(self, k):
                setattr(self, k, v)


    async def poll(self):
        measurements = []

        if not self.devices:
            await self._load_devices()

        tasks = [
            BLEInterface().poll_devices(self.devices),
            GPIOInterface().poll_devices(self.devices)
        ]

        # collect measurements
        results = await asyncio.gather(*tasks)

        # parse and normalize measurement data
        if results:
            measurements = Parser(**self._kwargs).parse(results)
            logger.debug("MEASUREMENTS: %s" % measurements)

        # send measurement data into the pipeline
        p = Pipeline()
        t = asyncio.create_task(p.input(measurements))
        self.devices = []

        return measurements


    async def poll_id(self, pk:int):
        """
        Poll measurements only for a named device id
        """

        device = await Device.get_by_id(pk)
        logger.debug(f"Loaded individual device: {device.as_dict()}")
        if device is not None:
            self.devices = [device]
            await self.poll()
            self.devices = []


    async def _load_devices(self):
        results = await Device.select().where(Device.room_id != None)
        for d in results:
            self.devices.append(d)



from quart_wtf import QuartForm
from wtforms.fields import (IntegerField, StringField, TextAreaField,
    HiddenField, SelectField)
from wtforms import validators


class DeviceForm(QuartForm):
    id = HiddenField('id', validators=[validators.optional()])
    interface = HiddenField('interface', validators=[validators.optional()])
    remove = HiddenField('remove', validators=[validators.optional()])
    name = StringField('name', validators=[validators.DataRequired()])
    room_id = SelectField('room')
    secret = StringField('secret', validators=[validators.optional()])



import asyncio
import logging
import json
from quart import (Blueprint, jsonify, render_template, abort, request, flash,
    redirect)
from quart_auth import login_required
from jinja2 import TemplateNotFound
from potnanny.models.schedule import Schedule, ScheduleSchema
from potnanny.database import db


logger = logging.getLogger(__name__)
bp = Blueprint('schedules', __name__)


@bp.route('/api/v1.0/schedules', methods=['GET'])
@login_required
async def get_list():
    objects = await Schedule.select()
    if not objects:
        return jsonify({"status": "error", "msg": "object not found"}), 404

    return jsonify({"status": "ok",
        "msg": [o.as_dict() for o in objects] }), 200


@bp.route('/api/v1.0/schedules', methods=['POST'])
@login_required
async def create():
    try:
        jsondata = await request.get_json()
        schema = ScheduleSchema()
        data = schema.load(jsondata)
        async with db.connection():
            obj = await Schedule.create(**data)
            await obj.save()
        await flash("Schedule created", "info")
        return jsonify({"status": "ok", "msg": obj.as_dict()}), 201
    except Exception as x:
        return jsonify({"status": "error", "msg": str(x) }), 500


@bp.route('/api/v1.0/schedules/<int:pk>', methods=['GET'])
@login_required
async def get(pk):
    obj = await Schedule.get_by_id(pk)
    if not obj:
        return jsonify(
            {"status": "error", "msg": f"object with id {pk} not found"}), 404

    return jsonify({"status": "ok", "msg": obj.as_dict()}), 200


@bp.route('/api/v1.0/schedules/<int:pk>', methods=['PATCH'])
@login_required
async def patch(pk):
    try:
        jsondata = await request.get_json()
        schema = ScheduleSchema()
        data = schema.load(jsondata)
        async with db.connection():
            obj = await Schedule.get_by_id(pk)
            for k, v in data.items():
                setattr(obj, k, v)
            await obj.save()

        return jsonify({"status": "ok", "msg": obj.as_dict()}), 200
    except Exception as x:
        logger.warning(str(x))
        return jsonify(
            {"status": "error", "msg": f"object with id {pk} not found"}), 404


@bp.route('/api/v1.0/schedules/<int:pk>', methods=['DELETE'])
@login_required
async def delete(pk):
    async with db.connection():
        try:
            obj = await Schedule.get_by_id(pk)
            await obj.delete_instance()
            await flash("Schedule deleted", "info")
        except:
            pass

    return jsonify({
        "status": "ok", "msg": ""}), 200



import asyncio
import logging
import datetime
from potnanny.utils import utcnow
from potnanny.models.schedule import Schedule


logger = logging.getLogger(__name__)


async def run_schedules(
    now:datetime.datetime = datetime.datetime.now().replace(second=0, microsecond=0)):
    """
    Run scheduled actions
    """

    tasks = []
    schedules = await Schedule.select()
    logger.debug(schedules)
    if not schedules:
        return

    for s in schedules:
        asyncio.create_task(s.run(now))
